@@ -1,8 +1,10 @@
 # Copyright 2008-2014 by Michiel de Hoon.  All rights reserved.
 # Revisions copyright 2008-2015 by Peter Cock. All rights reserved.
-# This code is part of the Biopython distribution and governed by its
-# license.  Please see the LICENSE file that should have been included
-# as part of this package.
+#
+# This file is part of the Biopython distribution and governed by your
+# choice of the "Biopython License Agreement" or the "BSD 3-Clause License".
+# Please see the LICENSE file that should have been included as part of this
+# package.
 
 """Parser for XML results returned by NCBI's Entrez Utilities.
 
@@ -86,7 +88,7 @@ class NoneElement:
             attributes = self.attributes
         except AttributeError:
             return "NoneElement"
-        return "NoneElement(attributes=%s)" % repr(attributes)
+        return "NoneElement(attributes=%r)" % attributes
 
 
 class IntegerElement(int):
@@ -110,7 +112,7 @@ class IntegerElement(int):
             attributes = self.attributes
         except AttributeError:
             return text
-        return "IntegerElement(%s, attributes=%s)" % (text, repr(attributes))
+        return "IntegerElement(%s, attributes=%r)" % (text, attributes)
 
 
 class StringElement(str):
@@ -133,30 +135,7 @@ class StringElement(str):
         attributes = self.attributes
         if not attributes:
             return text
-        return "StringElement(%s, attributes=%s)" % (text, repr(attributes))
-
-
-class UnicodeElement(str):
-    """NCBI Entrez XML element mapped to a unicode string."""
-
-    def __new__(cls, value, tag, attributes, key=None):
-        """Create a UnicodeElement."""
-        self = str.__new__(cls, value)
-        self.tag = tag
-        if key is None:
-            self.key = tag
-        else:
-            self.key = key
-        self.attributes = attributes
-        return self
-
-    def __repr__(self):
-        """Return a string representation of the object."""
-        text = str.__repr__(self)
-        attributes = self.attributes
-        if not attributes:
-            return text
-        return "UnicodeElement(%s, attributes=%s)" % (text, repr(attributes))
+        return "StringElement(%s, attributes=%r)" % (text, attributes)
 
 
 class ListElement(list):
@@ -178,7 +157,7 @@ class ListElement(list):
         attributes = self.attributes
         if not attributes:
             return text
-        return "ListElement(%s, attributes=%s)" % (text, repr(attributes))
+        return "ListElement(%s, attributes=%r)" % (text, attributes)
 
     def store(self, value):
         """Append an element to the list, checking tags."""
@@ -211,7 +190,7 @@ class DictionaryElement(dict):
         attributes = self.attributes
         if not attributes:
             return text
-        return "DictElement(%s, attributes=%s)" % (text, repr(attributes))
+        return "DictElement(%s, attributes=%r)" % (text, attributes)
 
     def store(self, value):
         """Add an entry to the dictionary, checking tags."""
@@ -282,8 +261,8 @@ class DataHandler:
 
     from Bio import Entrez
 
-    global_dtd_dir = os.path.join(str(Entrez.__path__[0]), "DTDs")
-    global_xsd_dir = os.path.join(str(Entrez.__path__[0]), "XSDs")
+    global_dtd_dir = os.path.join(Entrez.__path__[0], "DTDs")
+    global_xsd_dir = os.path.join(Entrez.__path__[0], "XSDs")
     local_dtd_dir = ""
     local_xsd_dir = ""
 
@@ -420,7 +399,7 @@ class DataHandler:
         self.parser = None
         if self.element is not None:
             # No more XML data, but there is still some unfinished business
-            raise CorruptedXMLError("Premature end of XML stream")
+            raise CorruptedXMLError("Premature end of data")
 
         # Send out the remaining records
         yield from records
@@ -494,8 +473,8 @@ class DataHandler:
         """Handle start of an XML element."""
         if tag in self.items:
             assert tag == "Item"
-            name = str(attrs["Name"])  # convert from Unicode
-            itemtype = str(attrs["Type"])  # convert from Unicode
+            name = attrs["Name"]
+            itemtype = attrs["Type"]
             del attrs["Type"]
             if itemtype == "Structure":
                 del attrs["Name"]
@@ -651,15 +630,11 @@ class DataHandler:
         self.attributes = None
         if tag in self.items:
             assert tag == "Item"
-            key = str(attributes["Name"])  # convert from Unicode
+            key = attributes["Name"]
             del attributes["Name"]
         else:
             key = tag
-        # Convert Unicode strings to plain strings if possible
-        try:
-            value = StringElement(value, tag, attributes, key)
-        except UnicodeEncodeError:
-            value = UnicodeElement(value, tag, attributes, key)
+        value = StringElement(value, tag, attributes, key)
         if element is None:
             self.record = element
         else:
@@ -705,7 +680,7 @@ class DataHandler:
         attributes = self.attributes
         self.attributes = None
         assert tag == "Item"
-        key = str(attributes["Name"])  # convert from Unicode
+        key = attributes["Name"]
         del attributes["Name"]
         if self.data:
             value = int("".join(self.data))
